import { ChartD3Service } from '../../services/chart.d3.service';
export class ChartEvents {
    constructor(primarySeries, anjHover, chartService, popover) {
        this.primarySeries = primarySeries;
        this.anjHover = anjHover;
        this.chartService = chartService;
        this.popover = popover;
    }
    // Component Event Handlers
    anjOverlayHoverHandler(svg, data, x, axisConfigs, seriesMeta) {
        this.clearTimeouts();
        const targetData = this.chartService.getTargetingData(data);
        const dataIndex = ChartD3Service.getIndexOfClosestPoint(x, targetData);
        if (this.activeIndex !== dataIndex) {
            this.setPopoverData(dataIndex, data, axisConfigs, seriesMeta);
            this.displayPopover(svg, this.activeIndex, true);
        }
    }
    anjSVGHoverHandler(targetEl, data, dataIndex, axisConfigs, seriesMeta, seriesIndex) {
        this.clearTimeouts();
        this.setPopoverData(dataIndex, [data], axisConfigs, seriesMeta, seriesIndex);
        this.displayPopover(targetEl);
    }
    anjStackedHoverHandler(svg, data, dataIndex, axisConfigs, seriesMeta) {
        this.clearTimeouts();
        if (this.activeIndex !== dataIndex) {
            this.setPopoverData(dataIndex, data, axisConfigs, seriesMeta);
            this.displayPopover(svg, this.activeIndex, true);
        }
    }
    anjEndHoverHandler(svg) {
        this.svgExitTimer = setTimeout(() => {
            this.hidePopover(svg);
        }, ChartEvents.popoverDebounceLength);
    }
    // Popover controllers
    displayPopover(svg, activeIndex, focus) {
        const popoverEl = (!isNaN(activeIndex)) ? this.chartService.getPopoverEl(svg, activeIndex) : svg;
        this.display(popoverEl);
        if (focus) {
            ChartD3Service.showFocus(svg, activeIndex);
        }
    }
    async display(target) {
        if (!target) {
            return;
        }
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    hidePopover(svg) {
        this.activeIndex = null;
        this.popover.close();
        if (svg) {
            ChartD3Service.hideFocus(svg);
        }
    }
    popoverHoverEnd(e) {
        this.clearTimeouts();
        const eventTargetPath = e.path.map(p => p.nodeName);
        const properPopoverExit = eventTargetPath.includes('ANJ-OVERLAY') && e.relatedTarget.nodeName !== 'ANJ-OVERLAY';
        if (e.relatedTarget.nodeName !== 'rect' && properPopoverExit) {
            this.popoverExitTimer = setTimeout(() => {
                this.hidePopover();
            }, ChartEvents.popoverDebounceLength);
        }
    }
    popoverHoverStart() {
        this.clearTimeouts();
    }
    setPopoverData(dataIndex, data, axisConfigs, seriesMeta, seriesIndex) {
        this.activeIndex = dataIndex;
        const seriesStyle = this.chartService.getPopoverSeriesStyle();
        const activeData = { data, axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta, seriesIndex };
        this.anjHover.emit(activeData);
    }
    clearTimeouts() {
        clearTimeout(this.svgExitTimer);
        clearTimeout(this.popoverExitTimer);
    }
}
ChartEvents.popoverDebounceLength = 150;
